@@ -159,31 +159,31 @@ typedef struct libxsmm_generated_code {
 const char* libxsmm_strerror(unsigned int i_error_code);
 
 /* @TODO change int based architecture value */
-void libxsmm_generator_dense_inlineasm(const char*                     i_file_out,
-                                       const char*                     i_routine_name,
-                                       const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                       const char*                     i_arch );
+void libxsmm_generator_gemm_inlineasm(const char*                     i_file_out,
+                                      const char*                     i_routine_name,
+                                      const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                      const char*                     i_arch );
 
 /* @TODO change int based architecture value */
-void libxsmm_generator_dense_directasm(const char*                     i_file_out,
-                                       const char*                     i_routine_name,
-                                       const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                       const char*                     i_arch );
+void libxsmm_generator_gemm_directasm(const char*                     i_file_out,
+                                      const char*                     i_routine_name,
+                                      const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                      const char*                     i_arch );
 
 /* @TODO change int based architecture value */
-void libxsmm_generator_dense_kernel( libxsmm_generated_code*         io_generated_code,
-                                     const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                     const char*                     i_arch );
+void libxsmm_generator_gemm_kernel( libxsmm_generated_code*         io_generated_code,
+                                    const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                    const char*                     i_arch );
 
 /* @TODO change int based architecture value */
-void libxsmm_generator_sparse( const char*                     i_file_out,
+void libxsmm_generator_spgemm( const char*                     i_file_out,
                                const char*                     i_routine_name,
                                const libxsmm_gemm_descriptor* i_xgemm_desc,
                                const char*                     i_arch,
                                const char*                     i_csc_file_in );
 
 /* @TODO change int based architecture value */
-void libxsmm_generator_sparse_kernel( libxsmm_generated_code*         io_generated_code,
+void libxsmm_generator_spgemm_kernel( libxsmm_generated_code*         io_generated_code,
                                       const libxsmm_gemm_descriptor* i_xgemm_desc,
                                       const char*                     i_arch,
                                       const unsigned int*             i_row_idx,
