@@ -7,9 +7,6 @@
 # define REAL_TYPE double
 #endif
 
-#define USE_PARALLEL
-
-
 
 void init(int seed, REAL_TYPE *LIBXSMM_RESTRICT dst, double scale, libxsmm_blasint nrows, libxsmm_blasint ncols, libxsmm_blasint ld)
 {
@@ -59,10 +56,6 @@ int main(int argc, char* argv[])
   { /* Tiled xGEMM */
     int i; double duration;
     unsigned long long start = libxsmm_timer_tick();
-#if defined(_OPENMP) && defined(USE_PARALLEL)
-#   pragma omp parallel
-#   pragma omp single
-#endif
     for (i = 0; i < nrepeat; ++i) {
       LIBXSMM_XOMPS_SYMBOL(REAL_TYPE)(&transa, &transb, &m, &n, &k, &alpha, a, &lda, b, &ldb, &beta, c, &ldc);
     }
