@@ -52,6 +52,9 @@
 #if defined(_WIN32)
 # include <Windows.h>
 #else
+# if !defined(LIBXSMM_INTERNAL_MAP)
+#   define LIBXSMM_INTERNAL_MAP MAP_PRIVATE
+# endif
 # include <sys/mman.h>
 # include <pthread.h>
 # include <unistd.h>
@@ -85,9 +88,13 @@
 # endif
 #endif
 
-/* allow external definition to enable testing */
+/* allow external definition to enable testing corner cases (exhausted registry space) */
 #if !defined(LIBXSMM_REGSIZE)
-# define LIBXSMM_REGSIZE 524288 /* 524287: Mersenne Prime number */
+# if defined(LIBXSMM_HASH_BASIC) /* consider larger registry to better deal with low-quality hash */
+#   define LIBXSMM_REGSIZE /*1048576*/524288 /* no Mersenne Prime number required, but POT number */
+# else
+#   define LIBXSMM_REGSIZE 524288 /* 524287: Mersenne Prime number (2^19-1) */
+# endif
 # define LIBXSMM_HASH_MOD(N, NPOT) LIBXSMM_MOD2(N, NPOT)
 #else
 # define LIBXSMM_HASH_MOD(N, NGEN) ((N) % (NGEN))
@@ -128,6 +135,19 @@ typedef struct LIBXSMM_RETARGETABLE internal_regentry {
 LIBXSMM_DEBUG(LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL unsigned int internal_ncollisions = 0;)
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL internal_regentry* internal_registry = 0;
 
+/** Helper macro determining the default prefetch strategy which is used for statically generated kernels. */
+#if defined(_WIN32) || defined(__CYGWIN__) /*TODO: account for calling convention; avoid passing six arguments*/
+# define INTERNAL_PREFETCH LIBXSMM_PREFETCH_NONE
+#elif defined(__MIC__) && (0 > LIBXSMM_PREFETCH) /* auto-prefetch (frontend) */
+# define INTERNAL_PREFETCH LIBXSMM_PREFETCH_AL2BL2_VIA_C
+#elif (0 > LIBXSMM_PREFETCH) /* auto-prefetch (frontend) */
+# define INTERNAL_PREFETCH LIBXSMM_PREFETCH_SIGONLY
+#endif
+#if !defined(INTERNAL_PREFETCH)
+# define INTERNAL_PREFETCH LIBXSMM_PREFETCH
+#endif
+
+LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_prefetch = LIBXSMM_MAX(INTERNAL_PREFETCH, 0);
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_target_arch = LIBXSMM_TARGET_ARCH_GENERIC;
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL const char* internal_target_archid = 0;
 
@@ -215,6 +235,59 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
 # define INTERNAL_FIND_CODE_CACHE_FINALIZE(DESCRIPTOR, RESULT)
 #endif
 
+#if (0 != LIBXSMM_JIT)
+# define INTERNAL_FIND_CODE_JIT(DESCRIPTOR, ENTRY) \
+  /* check if code generation or fix-up is needed, also check whether JIT is supported (CPUID) */ \
+  if (0 == internal_find_code_result.pmm && LIBXSMM_X86_AVX <= internal_target_arch) { \
+    /* instead of blocking others, a try-lock would allow to let others to fallback to BLAS (return 0) during lock-time */ \
+    INTERNAL_FIND_CODE_LOCK(lock, i); /* lock the registry entry */ \
+    /* re-read registry entry after acquiring the lock */ \
+    if (0 == diff) { \
+      internal_find_code_result = (ENTRY)->code; \
+      internal_find_code_result.imm &= ~LIBXSMM_HASH_COLLISION; \
+    } \
+    if (0 == internal_find_code_result.pmm) { /* double-check after acquiring the lock */ \
+      if (0 == diff) { \
+        /* found a conflict-free registry-slot, and attempt to build the kernel */ \
+        internal_build(DESCRIPTOR, &internal_find_code_result, &((ENTRY)->code_size)); \
+        if (0 != internal_find_code_result.pmm) { /* synchronize registry entry */ \
+          (ENTRY)->descriptor = *(DESCRIPTOR); \
+          INTERNAL_FIND_CODE_WRITE(ENTRY, internal_find_code_result.pmm); \
+        } \
+      } \
+      else { /* 0 != diff */ \
+        const unsigned int base = i; \
+        if (0 == diff0) { \
+          /* flag existing entry as collision */ \
+          /*const*/ void * /*const*/ code = (void*)((ENTRY)->code.imm | LIBXSMM_HASH_COLLISION); \
+          /* find new slot to store the code version */ \
+          const unsigned int index = LIBXSMM_HASH_MOD(LIBXSMM_HASH_VALUE(hash), LIBXSMM_REGSIZE); \
+          i = (index != i ? index : LIBXSMM_HASH_MOD(index + 1, LIBXSMM_REGSIZE)); \
+          i0 = i; /* keep starting point of free-slot-search in mind */ \
+          LIBXSMM_DEBUG(++internal_ncollisions;) \
+          INTERNAL_FIND_CODE_WRITE(ENTRY, code); /* fix-up existing entry */ \
+          diff0 = diff; /* no more fix-up */ \
+        } \
+        else { \
+          const unsigned int next = LIBXSMM_HASH_MOD(i + 1, LIBXSMM_REGSIZE); \
+          if (next != i0) { /* linear search for free slot */ \
+            i = next; \
+          } \
+          else { /* out of registry capacity (no free slot found) */ \
+            diff = 0; \
+          } \
+        } \
+        (ENTRY) -= base; /* recalculate base address */ \
+        (ENTRY) += i; \
+      } \
+    } \
+    INTERNAL_FIND_CODE_UNLOCK(lock); \
+  } \
+  else
+#else
+# define INTERNAL_FIND_CODE_JIT(DESCRIPTOR, ENTRY)
+#endif
+
 #define INTERNAL_FIND_CODE(DESCRIPTOR, ENTRY, HASH_FUNCTION, DIFF_FUNCTION) \
   internal_code internal_find_code_result; \
 { \
@@ -228,7 +301,6 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
     (ENTRY) += i; /* actual entry */ \
     do { \
       INTERNAL_FIND_CODE_READ(ENTRY, internal_find_code_result.pmm); /* read registered code */ \
-      /* entire block is conditional wrt LIBXSMM_JIT; static code currently does not have collisions */ \
       if (0 != internal_find_code_result.pmm) { \
         if (0 == diff0) { \
           if (0 == (LIBXSMM_HASH_COLLISION & internal_find_code_result.imm)) { /* check for no collision */ \
@@ -240,24 +312,28 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
               internal_find_code_result.pmm = 0; \
             } \
           } \
-          /* collision discovered but code version exists; perform initial deep check */ \
+          /* collision discovered but code version exists; perform deep check */ \
           else if (0 != (DIFF_FUNCTION)(DESCRIPTOR, &((ENTRY)->descriptor))) { \
             /* continue linearly searching code starting at re-hashed index position */ \
             const unsigned int index = LIBXSMM_HASH_MOD(LIBXSMM_HASH_VALUE(hash), LIBXSMM_REGSIZE); \
             unsigned int next; \
             internal_regentry *const registry = (ENTRY) - i; /* recalculate base address */ \
             for (i0 = (index != i ? index : LIBXSMM_HASH_MOD(index + 1, LIBXSMM_REGSIZE)), \
-              i = i0, next = LIBXSMM_HASH_MOD(i0 + 1, LIBXSMM_REGSIZE); next != i0/*no code found*/ && \
+              i = i0, next = LIBXSMM_HASH_MOD(i0 + 1, LIBXSMM_REGSIZE); \
               /* skip any (still invalid) descriptor which corresponds to no code, or continue on difference */ \
-              (0 == (ENTRY = (registry + i))->code.pmm || 0 != (diff = (DIFF_FUNCTION)(DESCRIPTOR, &((ENTRY)->descriptor)))); \
+              (0 == (ENTRY = (registry + i))->code.pmm || 0 != (diff = (DIFF_FUNCTION)(DESCRIPTOR, &((ENTRY)->descriptor)))) \
+                /* entire registry was searched and no code version was found */ \
+                && next != i0; \
               i = next, next = LIBXSMM_HASH_MOD(i + 1, LIBXSMM_REGSIZE)); \
             if (0 == diff) { /* found exact code version; continue with atomic load */ \
-              continue; \
+              internal_find_code_result.pmm = (ENTRY)->code.pmm; \
+              /* clear the uppermost bit of the address */ \
+              internal_find_code_result.imm &= ~LIBXSMM_HASH_COLLISION; \
             } \
             else { /* no code found */ \
               internal_find_code_result.pmm = 0; \
-              break; \
             } \
+            break; \
           } \
           else { /* clear the uppermost bit of the address */ \
             internal_find_code_result.imm &= ~LIBXSMM_HASH_COLLISION; \
@@ -267,52 +343,8 @@ LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_LOCK_TYPE internal_regl
           internal_find_code_result.pmm = 0; \
         } \
       } \
-      /* check if code generation or fix-up is needed, also check whether JIT is supported (CPUID) */ \
-      if (0 == internal_find_code_result.pmm && LIBXSMM_X86_AVX <= internal_target_arch) { \
-        INTERNAL_FIND_CODE_LOCK(lock, i); /* lock the registry entry */ \
-        /* re-read registry entry after acquiring the lock */ \
-        if (0 == diff) { \
-          internal_find_code_result = (ENTRY)->code; \
-          internal_find_code_result.imm &= ~LIBXSMM_HASH_COLLISION; \
-        } \
-        if (0 == internal_find_code_result.pmm) { /* double-check after acquiring the lock */ \
-          if (0 == diff) { \
-            /* found a conflict-free registry-slot, and attempt to build the kernel */ \
-            internal_build(DESCRIPTOR, &internal_find_code_result, &((ENTRY)->code_size)); \
-            if (0 != internal_find_code_result.pmm) { /* synchronize registry entry */ \
-              (ENTRY)->descriptor = *(DESCRIPTOR); \
-              INTERNAL_FIND_CODE_WRITE(ENTRY, internal_find_code_result.pmm); \
-            } \
-          } \
-          else { /* 0 != diff */ \
-            const unsigned int base = i; \
-            if (0 == diff0) { \
-              /* flag existing entry as collision */ \
-              /*const*/ void * /*const*/ code = (void*)((ENTRY)->code.imm | LIBXSMM_HASH_COLLISION); \
-              /* find new slot to store the code version */ \
-              const unsigned int index = LIBXSMM_HASH_MOD(LIBXSMM_HASH_VALUE(hash), LIBXSMM_REGSIZE); \
-              i = (index != i ? index : LIBXSMM_HASH_MOD(index + 1, LIBXSMM_REGSIZE)); \
-              i0 = i; /* keep starting point of free-slot-search in mind */ \
-              LIBXSMM_DEBUG(++internal_ncollisions;) \
-              INTERNAL_FIND_CODE_WRITE(ENTRY, code); /* fix-up existing entry */ \
-              diff0 = diff; /* no more fix-up */ \
-            } \
-            else { \
-              const unsigned int next = LIBXSMM_HASH_MOD(i + 1, LIBXSMM_REGSIZE); \
-              if (next != i0) { /* linear search for free slot */ \
-                i = next; \
-              } \
-              else { /* out of registry capacity (no free slot found) */ \
-                diff = 0; \
-              } \
-            } \
-            (ENTRY) -= base; /* recalculate base address */ \
-            (ENTRY) += i; \
-          } \
-        } \
-        INTERNAL_FIND_CODE_UNLOCK(lock); \
-      } \
-      else { \
+      INTERNAL_FIND_CODE_JIT(DESCRIPTOR, ENTRY) \
+      { \
         diff = 0; \
       } \
     } \
@@ -326,11 +358,12 @@ return internal_find_code_result.xmm
   INTERNAL_FIND_CODE_DECLARE(entry); \
   const signed char scalpha = (signed char)(0 == (PALPHA) ? LIBXSMM_ALPHA : *(PALPHA)), scbeta = (signed char)(0 == (PBETA) ? LIBXSMM_BETA : *(PBETA)); \
   if (0 == ((FLAGS) & (LIBXSMM_GEMM_FLAG_TRANS_A | LIBXSMM_GEMM_FLAG_TRANS_B)) && 1 == scalpha && (1 == scbeta || 0 == scbeta)) { \
+    const int internal_dispatch_main_prefetch = (0 == (PREFETCH) ? INTERNAL_PREFETCH : *(PREFETCH)); \
     DESCRIPTOR_DECL; LIBXSMM_GEMM_DESCRIPTOR(*(DESC), 0 != (VECTOR_WIDTH) ? (VECTOR_WIDTH): LIBXSMM_ALIGNMENT, FLAGS, LIBXSMM_LD(M, N), LIBXSMM_LD(N, M), K, \
       0 == LIBXSMM_LD(PLDA, PLDB) ? LIBXSMM_LD(M, N) : *LIBXSMM_LD(PLDA, PLDB), \
       0 == LIBXSMM_LD(PLDB, PLDA) ? (K) : *LIBXSMM_LD(PLDB, PLDA), \
       0 == (PLDC) ? LIBXSMM_LD(M, N) : *(PLDC), scalpha, scbeta, \
-      0 == (PREFETCH) ? LIBXSMM_PREFETCH : *(PREFETCH)); \
+      0 > internal_dispatch_main_prefetch ? internal_prefetch : internal_dispatch_main_prefetch); \
     { \
       INTERNAL_FIND_CODE(DESC, entry, HASH_FUNCTION, DIFF_FUNCTION).SELECTOR; \
     } \
@@ -360,6 +393,34 @@ return internal_find_code_result.xmm
   M, N, K, PLDA, PLDB, PLDC, PALPHA, PBETA, PREFETCH, dmm, HASH_FUNCTION, DIFF_FUNCTION)
 
 
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_register_static_code(
+  const libxsmm_gemm_descriptor* desc, unsigned int index, unsigned int hash, libxsmm_xmmfunction src,
+  internal_regentry* dst, unsigned int* registered, unsigned int* total)
+{
+  assert(0 != desc && 0 != src.dmm && 0 != dst && 0 != registered && 0 != total);
+  if (0 != dst->code.pmm) { /* collision? */
+    /* start at a re-hashed index position */
+    const unsigned int start = LIBXSMM_HASH_MOD(LIBXSMM_HASH_VALUE(hash), LIBXSMM_REGSIZE);
+    internal_regentry *const registry = dst - index;
+    unsigned int i0, i, next;
+
+    /* mark current entry as a collision (this might be already the case) */
+    dst->code.imm |= LIBXSMM_HASH_COLLISION;
+
+    /* start linearly searching for an available slot */
+    for (i = (start != index) ? start : LIBXSMM_HASH_MOD(start + 1, LIBXSMM_REGSIZE), i0 = i, next = LIBXSMM_HASH_MOD(i + 1, LIBXSMM_REGSIZE);
+      0 != (dst = registry + i)->code.pmm && next != i0; i = next, next = LIBXSMM_HASH_MOD(i + 1, LIBXSMM_REGSIZE));
+  }
+  if (0 == dst->code.pmm) { /* registry not (yet) exhausted */
+    dst->code.xmm = src;
+    dst->code_size = 0; /* statically generated code */
+    dst->descriptor = *desc;
+    ++(*registered);
+  }
+  ++(*total);
+}
+
+
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE internal_regentry* internal_init(void)
 {
   /*const*/internal_regentry* result;
@@ -427,10 +488,34 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE internal_regentry* internal_init(void)
       }
       if (0 == internal_target_archid) {
         internal_target_arch = libxsmm_cpuid_x86(&internal_target_archid);
+        assert(0 != internal_target_archid);
+      }
+      { /* select prefetch strategy for JIT */
+        const char *const env_prefetch = getenv("LIBXSMM_PREFETCH");
+        if (0 == env_prefetch || 0 == *env_prefetch) {
+          if (0 > LIBXSMM_PREFETCH) { /* permitted by LIBXSMM_PREFETCH_AUTO */
+            assert(0 != internal_target_archid);
+            internal_prefetch = 0 != strcmp("knl", internal_target_archid)
+              ? LIBXSMM_PREFETCH_NONE : LIBXSMM_PREFETCH_AL2BL2_VIA_C;
+          }
+        }
+        else { /* user input considered even if LIBXSMM_PREFETCH_AUTO is disabled */
+          switch (atoi(env_prefetch)) {
+            case 2: internal_prefetch = LIBXSMM_PREFETCH_SIGONLY; break;
+            case 3: internal_prefetch = LIBXSMM_PREFETCH_BL2_VIA_C; break;
+            case 4: internal_prefetch = LIBXSMM_PREFETCH_AL2; break;
+            case 5: internal_prefetch = LIBXSMM_PREFETCH_AL2_AHEAD; break;
+            case 6: internal_prefetch = LIBXSMM_PREFETCH_AL2BL2_VIA_C; break;
+            case 7: internal_prefetch = LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD; break;
+            case 8: internal_prefetch = LIBXSMM_PREFETCH_AL2_JPST; break;
+            case 9: internal_prefetch = LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST; break;
+            default: internal_prefetch = LIBXSMM_PREFETCH_NONE;
+          }
+        }
       }
       libxsmm_hash_init(internal_target_arch);
       libxsmm_gemm_diff_init(internal_target_arch);
-      init_code = libxsmm_gemm_init(internal_target_archid, 0/*auto-discovered*/, 0/*auto-discovered*/);
+      init_code = libxsmm_gemm_init(internal_target_archid, internal_prefetch);
 #if defined(__TRACE)
       const char *const env_trace_init = getenv("LIBXSMM_TRACE");
       if (EXIT_SUCCESS == init_code && 0 != env_trace_init) {
@@ -461,17 +546,19 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE internal_regentry* internal_init(void)
 #if (0 != LIBXSMM_JIT) && !defined(__MIC__)
           if (LIBXSMM_STATIC_TARGET_ARCH >= internal_target_arch)
 #endif
-          {
-            LIBXSMM_DEBUG(unsigned int csp = 0, cdp = 0;)
+          { /* opening a scope for eventually declaring variables */
+            unsigned int csp_tot = 0, csp_reg = 0, cdp_tot = 0, cdp_reg = 0;
             /* setup the dispatch table for the statically generated code */
 #           include <libxsmm_dispatch.h>
 #if !defined(NDEBUG) /* library code is expected to be mute */
-            if (0 < csp) {
-              fprintf(stderr, "LIBXSMM: %u SP-kernels are not registered due to hash key collisions!\n", csp);
+            if (csp_reg < csp_tot) {
+              fprintf(stderr, "LIBXSMM: %u of %u SP-kernels are not registered due to hash key collisions!\n", csp_tot - csp_reg, csp_tot);
             }
-            if (0 < cdp) {
-              fprintf(stderr, "LIBXSMM: %u DP-kernels are not registered due to hash key collisions!\n", cdp);
+            if (cdp_reg < cdp_tot) {
+              fprintf(stderr, "LIBXSMM: %u of %u DP-kernels are not registered due to hash key collisions!\n", cdp_tot - cdp_reg, cdp_tot);
             }
+#else
+            LIBXSMM_UNUSED(csp_tot); LIBXSMM_UNUSED(csp_reg); LIBXSMM_UNUSED(cdp_tot); LIBXSMM_UNUSED(cdp_reg);
 #endif
           }
           atexit(libxsmm_finalize);
@@ -591,11 +678,12 @@ LIBXSMM_RETARGETABLE void libxsmm_finalize(void)
           for (i = 0; i < LIBXSMM_REGSIZE; ++i) {
             const unsigned int code_size = registry[i].code_size;
             internal_code code = registry[i].code;
-            if (0 != code.pmm/*allocated*/) {
-              if (0 != code_size/*JIT*/) {
+            if (0 != code.pmm/*potentially allocated*/) {
+              if (0 != code_size/*JIT: actually allocated*/) {
                 /* make address valid by clearing an eventual collision flag */
                 code.imm &= ~LIBXSMM_HASH_COLLISION;
 #if defined(_WIN32)
+                /* TODO: executable memory buffer under Windows */
 #else
 # if defined(NDEBUG)
                 munmap(code.pmm, code_size);
@@ -636,13 +724,35 @@ LIBXSMM_RETARGETABLE void libxsmm_finalize(void)
 
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int libxsmm_get_target_arch()
 {
+#if !defined(_WIN32) && !defined(__MIC__) && (!defined(__CYGWIN__) || !defined(NDEBUG)/*code-coverage with Cygwin; fails@runtime!*/)
   return internal_target_arch;
+#else /* no JIT support */
+  return LIBXSMM_TARGET_ARCH_GENERIC;
+#endif
+}
+
+
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE const char* libxsmm_get_target_archid()
+{
+  return internal_target_archid;
+}
+
+
+/* function serves as a helper for implementing the Fortran interface */
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void get_target_archid(char* name, int length)
+{
+  const char* c = internal_target_archid ? internal_target_archid : "";
+  int i;
+  assert(0 != name); /* valid here since function is not in the public interface */
+  for (i = 0; i < length && 0 != *c; ++i, ++c) name[i] = *c;
+  for (; i < length; ++i) name[i] = ' ';
 }
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descriptor* desc, internal_code* code, unsigned int* code_size)
 {
-#if !defined(_WIN32) && !defined(__MIC__) && (!defined(__CYGWIN__) || !defined(NDEBUG)/*code-coverage with Cygwin; fails@runtime!*/)
+#if (0 != LIBXSMM_JIT)
+# if !defined(_WIN32) && !defined(__MIC__) && (!defined(__CYGWIN__) || !defined(NDEBUG)/*code-coverage with Cygwin; fails@runtime!*/)
   libxsmm_generated_code generated_code;
   assert(0 != desc && 0 != code && 0 != code_size);
   assert(0 != internal_target_archid);
@@ -660,31 +770,31 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
 
   /* handle an eventual error in the else-branch */
   if (0 == generated_code.last_error) {
-#if defined(__APPLE__) && defined(__MACH__)
+# if defined(__APPLE__) && defined(__MACH__)
     const int fd = 0;
-#else
+# else
     const int fd = open("/dev/zero", O_RDWR);
-#endif
+# endif
     if (0 <= fd) {
       /* create executable buffer */
       code->pmm = mmap(0, generated_code.code_size,
         /* must be a superset of what mprotect populates (see below) */
         PROT_READ | PROT_WRITE | PROT_EXEC,
-#if defined(__APPLE__) && defined(__MACH__)
-        MAP_ANON | MAP_PRIVATE, fd, 0);
-#elif !defined(__CYGWIN__)
-        MAP_PRIVATE | MAP_32BIT, fd, 0);
+# if defined(__APPLE__) && defined(__MACH__)
+        LIBXSMM_INTERNAL_MAP | MAP_ANON, fd, 0);
+# elif !defined(__CYGWIN__)
+        LIBXSMM_INTERNAL_MAP | MAP_32BIT, fd, 0);
       close(fd);
-#else
-        MAP_PRIVATE, fd, 0);
+# else
+        LIBXSMM_INTERNAL_MAP, fd, 0);
       close(fd);
-#endif
+# endif
       if (MAP_FAILED != code->pmm) {
         /* explicitly disable THP for this memory region, kernel 2.6.38 or higher */
-#if defined(MADV_NOHUGEPAGE)
-# if defined(NDEBUG)
+# if defined(MADV_NOHUGEPAGE)
+#  if defined(NDEBUG)
         madvise(code->pmm, generated_code.code_size, MADV_NOHUGEPAGE);
-# else /* library code is expected to be mute */
+#  else /* library code is expected to be mute */
         /* proceed even in case of an error, we then just take what we got (THP) */
         if (0 != madvise(code->pmm, generated_code.code_size, MADV_NOHUGEPAGE)) {
           static LIBXSMM_TLS int once = 0;
@@ -695,17 +805,17 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
             once = 1;
           }
         }
-# endif /*defined(NDEBUG)*/
-#elif !(defined(__APPLE__) && defined(__MACH__)) && !defined(__CYGWIN__)
+#  endif /*defined(NDEBUG)*/
+# elif !(defined(__APPLE__) && defined(__MACH__)) && !defined(__CYGWIN__)
         LIBXSMM_MESSAGE("================================================================================")
         LIBXSMM_MESSAGE("LIBXSMM: Adjusting THP is unavailable due to C89 or kernel older than 2.6.38!")
         LIBXSMM_MESSAGE("================================================================================")
-#endif /*MADV_NOHUGEPAGE*/
+# endif /*MADV_NOHUGEPAGE*/
         /* copy temporary buffer into the prepared executable buffer */
         memcpy(code->pmm, generated_code.generated_code, generated_code.code_size);
 
         if (0/*ok*/ == mprotect(code->pmm, generated_code.code_size, PROT_EXEC | PROT_READ)) {
-#if !defined(NDEBUG) && defined(_DEBUG)
+# if !defined(NDEBUG) && defined(_DEBUG)
           /* write buffer for manual decode as binary to a file */
           char objdump_name[512];
           FILE* byte_code;
@@ -721,16 +831,16 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
             fwrite(generated_code.generated_code, 1, generated_code.code_size, byte_code);
             fclose(byte_code);
           }
-#endif /*!defined(NDEBUG) && defined(_DEBUG)*/
+# endif /*!defined(NDEBUG) && defined(_DEBUG)*/
           /* free temporary/initial code buffer */
           free(generated_code.generated_code);
           /* finalize code generation */
           *code_size = generated_code.code_size;
         }
         else { /* there was an error with mprotect */
-#if defined(NDEBUG)
+# if defined(NDEBUG)
           munmap(code->pmm, generated_code.code_size);
-#else /* library code is expected to be mute */
+# else /* library code is expected to be mute */
           static LIBXSMM_TLS int once = 0;
           if (0 == once) {
             const int error = errno;
@@ -747,12 +857,12 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
               once_mmap_error = 1;
             }
           }
-#endif
+# endif
           free(generated_code.generated_code);
         }
       }
       else {
-#if !defined(NDEBUG) /* library code is expected to be mute */
+# if !defined(NDEBUG) /* library code is expected to be mute */
         static LIBXSMM_TLS int once = 0;
         if (0 == once) {
           const int error = errno;
@@ -760,13 +870,13 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
             strerror(error), error);
           once = 1;
         }
-#endif
+# endif
         free(generated_code.generated_code);
         /* clear MAP_FAILED value */
         code->pmm = 0;
       }
     }
-#if !defined(NDEBUG)/* library code is expected to be mute */
+# if !defined(NDEBUG)/* library code is expected to be mute */
     else {
       static LIBXSMM_TLS int once = 0;
       if (0 == once) {
@@ -774,27 +884,30 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_build(const libxsmm_gemm_descr
         once = 1;
       }
     }
-#endif
+# endif
   }
   else {
-#if !defined(NDEBUG) /* library code is expected to be mute */
+# if !defined(NDEBUG) /* library code is expected to be mute */
     static LIBXSMM_TLS int once = 0;
     if (0 == once) {
       fprintf(stderr, "%s (error #%u)\n", libxsmm_strerror(generated_code.last_error),
         generated_code.last_error);
       once = 1;
     }
-#endif
+# endif
     free(generated_code.generated_code);
   }
-#elif !defined(__MIC__)
-  LIBXSMM_UNUSED(desc); LIBXSMM_UNUSED(code); LIBXSMM_UNUSED(code_size);
+# else
+#   if !defined(__MIC__)
   LIBXSMM_MESSAGE("================================================================================")
   LIBXSMM_MESSAGE("LIBXSMM: The JIT BACKEND is currently not supported under Microsoft Windows!")
   LIBXSMM_MESSAGE("================================================================================")
-#else
+#   endif
   LIBXSMM_UNUSED(desc); LIBXSMM_UNUSED(code); LIBXSMM_UNUSED(code_size);
-#endif /*_WIN32*/
+  /* libxsmm_get_target_arch also serves as a runtime check whether JIT is available or not */
+  assert(LIBXSMM_X86_AVX > libxsmm_get_target_arch());
+# endif /*_WIN32*/
+#endif /*LIBXSMM_JIT*/
 }
 
 
