@@ -41,6 +41,7 @@
 #endif
 #include <stdlib.h>
 #include <stdint.h>
+#include <string.h>
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(pop)
 #endif
@@ -60,50 +61,75 @@ LIBXSMM_RETARGETABLE libxsmm_dgemm_function libxsmm_internal_dgemm = LIBXSMM_FSY
 
 
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_tile_sizes[/*configs*/][2/*DP/SP*/][3/*TILE_M,TILE_N,TILE_K*/] = {
-  { { 128, 48, 48 }, { 64, 48, 80 } }, /*generic*/
-  { { 128, 48, 48 }, { 64, 48, 80 } }  /*knl*/
+  { { 72, 32, 16 }, { 72, 32, 16 } }, /*generic*/
+  { { 72, 32, 16 }, { 72, 32, 16 } }  /*knl*/
 };
 LIBXSMM_RETARGETABLE int libxsmm_internal_tile_size[/*DP/SP*/][3/*TILE_M,TILE_N,TILE_K*/] = {
   { 0, 0, 0 }, { 0, 0, 0 }
 };
-LIBXSMM_RETARGETABLE int libxsmm_internal_num_nt = 2;
+LIBXSMM_RETARGETABLE int libxsmm_internal_gemm_prefetch = LIBXSMM_MAX(LIBXSMM_PREFETCH, 0);
+LIBXSMM_RETARGETABLE int libxsmm_internal_gemm_nthreads_per_core = 2;
+LIBXSMM_RETARGETABLE int libxsmm_internal_gemm_tasks = 0;
+LIBXSMM_RETARGETABLE int libxsmm_internal_gemm_omp = 2;
 LIBXSMM_RETARGETABLE int libxsmm_internal_gemm = 0;
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_gemm_configure(const char* archid, int gemm_kind)
-{
-  const int config = (0 == archid || 'k' != archid[0] || 'n' != archid[1] || 'l' != archid[2]) ? 0 : 1;
-  const char* env[3], *const env_gemm_kind = getenv("LIBXSMM_GEMM");
-
-  /* determine what will be executed in the wrapper code (0: small gemm, 1: sequential, 2: parallelized) */
-  libxsmm_internal_gemm = (env_gemm_kind ? atoi(env_gemm_kind) : gemm_kind);
-  libxsmm_internal_num_nt = 1 == config ? 4 : 2; /* threads per core */
-
-  /* attempt to setup tile sizes from the environment (LIBXSMM_TILEM, LIBXSMM_TILEN, and LIBXSMM_TILEK) */
-  env[0] = getenv("LIBXSMM_TILEM"); env[1] = getenv("LIBXSMM_TILEN"); env[2] = getenv("LIBXSMM_TILEK");
-  libxsmm_internal_tile_size[0/*DP*/][0/*M*/] = (env[0] ? atoi(env[0]) : 0);
-  libxsmm_internal_tile_size[0/*DP*/][1/*N*/] = (env[1] ? atoi(env[1]) : 0);
-  libxsmm_internal_tile_size[0/*DP*/][2/*K*/] = (env[2] ? atoi(env[2]) : 0);
-  /* environment-defined tile sizes applies for DP and SP */
-  libxsmm_internal_tile_size[1/*SP*/][0/*M*/] = libxsmm_internal_tile_size[0/*DP*/][0];
-  libxsmm_internal_tile_size[1/*SP*/][1/*N*/] = libxsmm_internal_tile_size[0/*DP*/][1];
-  libxsmm_internal_tile_size[1/*SP*/][2/*K*/] = libxsmm_internal_tile_size[0/*DP*/][2];
-
-  /* load predefined configuration if tile size is not setup by the environment */
-  if (0 >= libxsmm_internal_tile_size[0/*DP*/][0/*M*/]) libxsmm_internal_tile_size[0][0] = internal_tile_sizes[config][0][0];
-  if (0 >= libxsmm_internal_tile_size[0/*DP*/][1/*N*/]) libxsmm_internal_tile_size[0][1] = internal_tile_sizes[config][0][1];
-  if (0 >= libxsmm_internal_tile_size[0/*DP*/][2/*K*/]) libxsmm_internal_tile_size[0][2] = internal_tile_sizes[config][0][2];
-  if (0 >= libxsmm_internal_tile_size[1/*SP*/][0/*M*/]) libxsmm_internal_tile_size[1][0] = internal_tile_sizes[config][1][0];
-  if (0 >= libxsmm_internal_tile_size[1/*SP*/][1/*N*/]) libxsmm_internal_tile_size[1][1] = internal_tile_sizes[config][1][1];
-  if (0 >= libxsmm_internal_tile_size[1/*SP*/][2/*K*/]) libxsmm_internal_tile_size[1][2] = internal_tile_sizes[config][1][2];
-}
-
-
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_GEMM_WEAK_DLIB int libxsmm_gemm_init(const char* archid,
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_gemm_configure(const char* archid, int prefetch,
   libxsmm_sgemm_function sgemm_function, libxsmm_dgemm_function dgemm_function)
 {
-  /* internal pre-initialization step */
-  libxsmm_gemm_configure(archid, 0/*default gemm kind is small gemm*/);
+  int config = 0;
+
+  libxsmm_internal_gemm_prefetch = LIBXSMM_PREFETCH_AL2_AHEAD;
+  LIBXSMM_UNUSED(prefetch);
+
+#if !defined(__MIC__)
+  if (0 == strcmp("knl", archid))
+#endif
+  {
+    libxsmm_internal_gemm_nthreads_per_core = 4;
+    config = 1;
+  }
+
+  { /* behaviour of libxsmm_omp_?gemm routines (OpenMP based)
+     * 1: parallelized but without internal parallel region,
+     * 2: parallelized with internal parallel region" )
+     */
+    const char *const env = getenv("LIBXSMM_OMP");
+    if (0 != env && 0 != *env) {
+      libxsmm_internal_gemm_omp = atoi(env);
+    }
+  }
+
+  { /* behaviour of LD_PRELOAD ?GEMM routines
+     * 0: sequential below-threshold routine (no OpenMP) with fallback to BLAS,
+     * 1: OpenMP-parallelized but without internal parallel region,
+     * 2: OpenMP-parallelized with internal parallel region" )
+     */
+    const char *const env = getenv("LIBXSMM_GEMM");
+    if (0 != env && 0 != *env) {
+      libxsmm_internal_gemm = atoi(env);
+    }
+  }
+
+  { /* attempt to setup tile sizes from the environment (LIBXSMM_M, LIBXSMM_N, and LIBXSMM_K) */
+    const char* env[3];
+    env[0] = getenv("LIBXSMM_M"); env[1] = getenv("LIBXSMM_N"); env[2] = getenv("LIBXSMM_K");
+    libxsmm_internal_tile_size[0/*DP*/][0/*M*/] = (env[0] ? atoi(env[0]) : 0);
+    libxsmm_internal_tile_size[0/*DP*/][1/*N*/] = (env[1] ? atoi(env[1]) : 0);
+    libxsmm_internal_tile_size[0/*DP*/][2/*K*/] = (env[2] ? atoi(env[2]) : 0);
+    /* environment-defined tile sizes applies for DP and SP */
+    libxsmm_internal_tile_size[1/*SP*/][0/*M*/] = libxsmm_internal_tile_size[0/*DP*/][0];
+    libxsmm_internal_tile_size[1/*SP*/][1/*N*/] = libxsmm_internal_tile_size[0/*DP*/][1];
+    libxsmm_internal_tile_size[1/*SP*/][2/*K*/] = libxsmm_internal_tile_size[0/*DP*/][2];
+
+    /* load predefined configuration if tile size is not setup by the environment */
+    if (0 >= libxsmm_internal_tile_size[0/*DP*/][0/*M*/]) libxsmm_internal_tile_size[0][0] = internal_tile_sizes[config][0][0];
+    if (0 >= libxsmm_internal_tile_size[0/*DP*/][1/*N*/]) libxsmm_internal_tile_size[0][1] = internal_tile_sizes[config][0][1];
+    if (0 >= libxsmm_internal_tile_size[0/*DP*/][2/*K*/]) libxsmm_internal_tile_size[0][2] = internal_tile_sizes[config][0][2];
+    if (0 >= libxsmm_internal_tile_size[1/*SP*/][0/*M*/]) libxsmm_internal_tile_size[1][0] = internal_tile_sizes[config][1][0];
+    if (0 >= libxsmm_internal_tile_size[1/*SP*/][1/*N*/]) libxsmm_internal_tile_size[1][1] = internal_tile_sizes[config][1][1];
+    if (0 >= libxsmm_internal_tile_size[1/*SP*/][2/*K*/]) libxsmm_internal_tile_size[1][2] = internal_tile_sizes[config][1][2];
+  }
 
   if (NULL != sgemm_function) {
     libxsmm_internal_sgemm = sgemm_function;
@@ -128,6 +154,13 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_GEMM_WEAK_DLIB int libxsmm_gemm_in
     libxsmm_internal_dgemm = LIBXSMM_FSYMBOL(__real_mkl_dgemm);
   }
 #endif /*defined(LIBXSMM_GEMM_EXTWRAP)*/
+}
+
+
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_GEMM_WEAK_DLIB int libxsmm_gemm_init(const char* archid, int prefetch)
+{
+  /* internal pre-initialization step */
+  libxsmm_gemm_configure(archid, prefetch, 0/*auto-discovered*/, 0/*auto-discovered*/);
 
   return (NULL != libxsmm_internal_sgemm
        && NULL != libxsmm_internal_dgemm)
