@@ -32,11 +32,16 @@ int main()
   if (size != nerrors) {
     return size == i ? EXIT_SUCCESS : (i + 1)/*EXIT_FAILURE*/;
   }
-  else { /* potentially unsupported platforms (due to calling convention)
-          * or environment variable LIBXSMM_JIT is set to zero */
-    fprintf(stderr, "JIT support is potentially unavailable\n");
+  else if (LIBXSMM_X86_AVX > libxsmm_get_target_arch()) {
+    /* potentially unsupported platforms due to not supporting AVX, due to calling convention,
+     * or due to the environment variable LIBXSMM_JIT being set to zero.
+     */
+    fprintf(stderr, "JIT support is unavailable\n");
     return EXIT_SUCCESS;
   }
+  else {
+    return EXIT_FAILURE;
+  }
 #else
   fprintf(stderr, "Please rebuild LIBXSMM with JIT=1\n");
   return EXIT_SUCCESS;
